/*
 * checkpoint.c
 *
 * file format:
 *   int     = total number of atoms (32bit)
 *   int     = P = number of mpi processes when saved (or zero) (32bit)
 *   P x int = number of particles for each mpi process (32bit integers)
 *   then the atom data, for each atom: 
 *          double q[3], double p[3], long long tag, long long id.
 *
 * lj - mpi/c version - rvz/5 Oct 2010
 */

#include "checkpoint.h"

void cp_write(struct Atom* a, int n, MPI_Datatype t, char* f, MPI_Comm c) 
{
    int p,r;
    MPI_Comm_size(c,&p);
    MPI_Comm_rank(c,&r);
    int header[p+2];
    MPI_Allgather(&n,1,MPI_INT,&(header[2]),1,MPI_INT,c);
    int i,n_below=0;
    for(i=0;i<r;i++)
        n_below+=header[i+2];
    MPI_File h;
    MPI_File_open(c,f,MPI_MODE_CREATE|MPI_MODE_WRONLY,MPI_INFO_NULL,&h);
    if(r==p-1){
        header[0]=n_below+n;
        header[1]=p;
        MPI_File_write(h,header,p+2,MPI_INT,MPI_STATUS_IGNORE);
    }
    MPI_File_set_view(h,(p+2)*sizeof(int),t,t,"native",MPI_INFO_NULL);
    MPI_File_write_at_all(h,n_below,a,n,t,MPI_STATUS_IGNORE);
    MPI_File_close(&h);
}

void cp_read(struct Atom* a, int* n, int tot, MPI_Datatype t, char* f, MPI_Comm c)
{
    int p,r,i,n_below=0; 
    MPI_Comm_size(c,&p);
    MPI_Comm_rank(c,&r);
    int header[p+2];
    MPI_File h;
    MPI_File_open(c,f,MPI_MODE_RDONLY,MPI_INFO_NULL,&h);
    if(r==0)
        MPI_File_read(h,header,2,MPI_INT,MPI_STATUS_IGNORE);
    MPI_Bcast(header,2,MPI_INT,0,c);
    if(header[0]!=tot) 
       MPI_Abort(c,1);
    if(p==header[1]){ 
        if(r==0) 
            MPI_File_read(h,header+2,p,MPI_INT,MPI_STATUS_IGNORE);
        MPI_Bcast(header+2,p,MPI_INT,0,c);
        for(i=0;i<r;i++) 
            n_below+=header[i+2];
        *n=header[r+2];
    }else{
        n_below=r*(tot/p);
        *n=(tot/p);
        if(r==p-1) *n+=tot%p;
    }
    MPI_File_set_view(h,(header[1]+2)*sizeof(int),t,t,"native",MPI_INFO_NULL);
    MPI_File_read_at_all(h,n_below,a,*n,t,MPI_STATUS_IGNORE);
    MPI_File_close(&h);
}

