#include <mpi.h>
#include <stdio.h>
#include "latgen.h"
#include "ljtypes.h"

#define eps 1e-8

void init(struct Generator* gen, double r1[3], double r2[3], double l, double a)
{
    gen->origin[X] = r1[X];
    gen->origin[Y] = r1[Y];
    gen->origin[Z] = r1[Z];
    gen->end[X] = r2[X];
    gen->end[Y] = r2[Y];
    gen->end[Z] = r2[Z];
    gen->L = l;
    gen->a = a;
    gen->n = 0;
    gen->ix = -1;
    gen->iy = 0;
    gen->iz = 0;
    gen->r[X] = 0;
    gen->r[Y] = 0;
    gen->r[Z] = 0;
}


int make_lattice_position(struct Generator* gen)
{
    do {
        ++(gen->ix);   
        if (gen->ix*gen->a > gen->L - eps) {  
            gen->ix = 0;     
            ++(gen->iy);     
            if (gen->iy*gen->a > gen->L - eps) {  
                gen->iy = 0;     
                ++(gen->iz);     
                if (gen->iz*gen->a > gen->L - eps)  {  
                    fprintf(stderr,"ERROR: Ran out of lattice positions");
                    MPI_Abort(MPI_COMM_WORLD, 4);
                }
            }
        }
         
        gen->r[X] = gen->ix*gen->a - 0.5*gen->L;
        gen->r[Y] = gen->iy*gen->a - 0.5*gen->L;
        gen->r[Z] = gen->iz*gen->a - 0.5*gen->L;
        (gen->n)++;
    } while (gen->r[X] < gen->origin[X] || gen->r[X] >= gen->end[X] - eps ||
             gen->r[Y] < gen->origin[Y] || gen->r[Y] >= gen->end[Y] - eps ||
             gen->r[Z] < gen->origin[Z] || gen->r[Z] >= gen->end[Z] - eps);

    return gen->n-1;
}


int count_lattice_positions(struct Generator* gen) 
{
    int i = 0;
    int j = 0;
    int k = 0;
    int n = 0;
    while (k < (int)(gen->L/gen->a)) {
        double lat[3] = {i*gen->a - 0.5*gen->L,
                         j*gen->a - 0.5*gen->L,
                         k*gen->a - 0.5*gen->L};
        if (lat[X] >= gen->origin[X] && lat[X] < gen->end[X] - eps &&
            lat[Y] >= gen->origin[Y] && lat[Y] < gen->end[Y] - eps &&
            lat[Z] >= gen->origin[Z] && lat[Z] < gen->end[Z] - eps) {
            ++n;
        }
        ++i;
        if (i*gen->a > gen->L - eps) {
            i = 0;
            ++j;
            if (j*gen->a > gen->L - eps) { 
                j = 0;
                ++k;
            }
        }
    }

    return n;
}

