#include <mpi.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include "parameters.h"
#include "mpiglobal.h"

MPI_Aint get_mpi_address(void *location);

MPI_Datatype MPI_PARAMETERS;

void Parameters_commit()
{
    static int Parameters_commited = 0;
    if (Parameters_commited == 0) {
        struct Parameters p;
        int len[10] = {1,1,1,1,1,1,1,1,1,MAXCPFILENAMELEN};
        MPI_Aint addr[11] = {get_mpi_address(&p),
                             get_mpi_address(&p.n_tot),
                             get_mpi_address(&p.seed),
                             get_mpi_address(&p.side),
                             get_mpi_address(&p.dt),
                             get_mpi_address(&p.runtime),
                             get_mpi_address(&p.temp),
                             get_mpi_address(&p.burn_in),
                             get_mpi_address(&p.per),
                             get_mpi_address(&p.cp_interval),
                             get_mpi_address(p.cp_filename) };
        MPI_Aint disp[10] = {addr[1]-addr[0],addr[2]-addr[0],
                             addr[3]-addr[0],addr[4]-addr[0],
                             addr[5]-addr[0],addr[6]-addr[0],
                             addr[7]-addr[0],addr[8]-addr[0],
                             addr[9]-addr[0],addr[10]-addr[0]};
        MPI_Datatype t[10] ={MPI_INT,
                             MPI_LONG,
                             MPI_DOUBLE,
                             MPI_DOUBLE,
                             MPI_DOUBLE,
                             MPI_DOUBLE,
                             MPI_DOUBLE,
                             MPI_INT,
                             MPI_DOUBLE,
                             MPI_CHAR};
        MPI_Type_create_struct(10,len,disp,t,&MPI_PARAMETERS);
        MPI_Type_commit(&MPI_PARAMETERS);
        Parameters_commited = 1;
    }
}

void read_parameters(struct Parameters* this, MPI_Comm comm, FILE* in) 
{

    if (rank == root) {  
        double rho;
        if (in==stdin) {
            printf("#number of atoms:");
            fscanf(in, "%d", &(this->n_tot));
            printf("#density:");
            fscanf(in, "%lf", &rho);
            printf("#initial temperature:");
            fscanf(in, "%lf", &(this->temp));
            printf("#runtime:");
            fscanf(in, "%lf", &(this->runtime));
            printf("#time step:");
            fscanf(in, "%lf", &(this->dt));
            printf("#random seed:");
            fscanf(in, "%ld", &(this->seed)); 
            printf("#burn-in time:");
            fscanf(in, "%lf", &(this->burn_in));
            printf("#how often to write to xsf:");
            fscanf(in, "%d", &(this->per));
            printf("#checkpoint interval time:");
            fscanf(in, "%lf", &(this->cp_interval));
            printf("#checkpoint filename:");
            fscanf(in, "%s", this->cp_filename);
        } else {
            fscanf(in, "%d", &(this->n_tot));
            fscanf(in, "%lf", &rho);
            fscanf(in, "%lf", &(this->temp));
            fscanf(in, "%lf", &(this->runtime));
            fscanf(in, "%lf", &(this->dt));
            fscanf(in, "%ld", &(this->seed)); 
            fscanf(in, "%lf", &(this->burn_in));
            fscanf(in, "%d",  &(this->per));
            fscanf(in, "%lf", &(this->cp_interval));
            fscanf(in, "%s", this->cp_filename);
        }
         
        this->side = cbrt(this->n_tot/rho);
         
        printf(
            "# N = %d\n"
            "# L = %lf\n"
            "# T = %lf\n"
            "# runtime = %lf\n"
            "# dt = %lf\n"
            "# seed = %ld\n"
            "# burn_in = %lf\n"
            "# per = %d\n"
            "# cp_interval = %lf\n"
            "# cp_filename = %s\n",
            this->n_tot,
            this->side,	
            this->temp,
            this->runtime,
            this->dt ,
            this->seed,
            this->burn_in,
            this->per,
            this->cp_interval,
            this->cp_filename
            );
    }
     
    Parameters_commit();

    MPI_Bcast(this, 1, MPI_PARAMETERS, root, comm);
     
    this->seed += rank;  
}

